<?php

namespace Bixo\Opportunity\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Opportunity\Forms\Opportunity as OpportunityForm;
use Bixo\Opportunity\Http\Requests\OpportunityResourceRequest;
use Bixo\Opportunity\Http\Resources\OpportunityResource;
use Bixo\Opportunity\Http\Resources\OpportunitiesCollection;
use Bixo\Opportunity\Models\Opportunity;
use Bixo\Opportunity\Actions\OpportunityAction;
use Bixo\Opportunity\Actions\OpportunityActions;


/**
 * Resource controller class for opportunity.
 */
class OpportunityResourceController extends BaseController
{

    /**
     * Initialize opportunity resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = OpportunityForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.opportunity.modules'), 'opportunity', guard_url('opportunity'));
            return $next($request);
        });
    }

    /**
     * Display a list of opportunity.
     *
     * @return Response
     */
    public function index(OpportunityResourceRequest $request)
    {
        $request = $request->all();
        $page = OpportunityActions::run('paginate', $request);

        $data = new OpportunitiesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('opportunity::opportunity.names'))
            ->view('opportunity::opportunity.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display opportunity.
     *
     * @param Request $request
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function show(OpportunityResourceRequest $request, Opportunity $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OpportunityResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('opportunity::opportunity.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('opportunity::opportunity.show')
            ->output();
    }

    /**
     * Show the form for creating a new opportunity.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(OpportunityResourceRequest $request, Opportunity $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OpportunityResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('opportunity::opportunity.name'))
            ->view('opportunity::opportunity.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new opportunity.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(OpportunityResourceRequest $request, Opportunity $model)
    {
        try {
            $request = $request->all();
            $model = OpportunityAction::run('store', $model, $request);
            $data = new OpportunityResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('opportunity::opportunity.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('opportunity/opportunity/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/opportunity/opportunity'))
                ->redirect();
        }

    }

    /**
     * Show opportunity for editing.
     *
     * @param Request $request
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function edit(OpportunityResourceRequest $request, Opportunity $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OpportunityResource($model);
        // return view('opportunity::opportunity.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('opportunity::opportunity.name'))
            ->view('opportunity::opportunity.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the opportunity.
     *
     * @param Request $request
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function update(OpportunityResourceRequest $request, Opportunity $model)
    {
        try {
            $request = $request->all();
            $model = OpportunityAction::run('update', $model, $request);
            $data = new OpportunityResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('opportunity::opportunity.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('opportunity/opportunity/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/opportunity/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the opportunity.
     *
     * @param Model   $opportunity
     *
     * @return Response
     */
    public function destroy(OpportunityResourceRequest $request, Opportunity $model)
    {
        try {

            $request = $request->all();
            $model = OpportunityAction::run('destroy', $model, $request);
            $data = new OpportunityResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('opportunity::opportunity.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('opportunity/opportunity/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('opportunity/opportunity/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
